import re
import sys

from mapnik_utils.version_adapter import Mapnik

mapnik = Mapnik()

# mapnik_utils
from mapnik_utils.projection import EasyProjection

if not hasattr(mapnik,'ProjTransform'):
    from compatibility import ProjTransform

BoostPythonMetaclass = mapnik.Coord.__class__
                
class _injector(object):
    class __metaclass__(BoostPythonMetaclass):
        def __init__(self, name, bases, dict):
            for b in bases:
                if type(b) not in (self, type):
                    for k,v in dict.items():
                        setattr(b,k,v)
            return type.__init__(self, name, bases, dict)

if not hasattr(mapnik,'Box2d'):
    mapnik.Box2d = mapnik.Envelope

class _Map(mapnik.Map,_injector):

    def set_easy_srs(self,srs):
        self.srs = EasyProjection(srs).params()

    @property
    def proj_obj(self):
        return EasyProjection(self.srs)

    def lon_lat_bbox(self):
        return self.envelope().forward(self.proj_obj,EasyProjection(4326))

    def find_layer(self,name):
        lyr = [l for l in self.layers if l.name.lower() == name.lower()]
        if not lyr:
            raise ValueError('Layer "%s" not found, available layers are: ["%s"]' % (name,', '.join(self.layer_names())))
        return lyr[0]
    
    def layer_names(self):
        return [l.name for l in self.layers]

    def active_layers(self):
        return [l.name for l in self.layers if l.active]

    def zoom_to_layer(self,layer):
        layer = self.find_layer(layer)
        layer_box = layer.envelope()
        box = layer_box.forward(layer.proj_obj,self.proj_obj)
        self.zoom_to_box(box)

    def lon_lat_layers_bounds(self):
        return self.layers_bounds().forward(self.proj_obj,EasyProjection(4326))

    def layers_bounds(self):
        new_box = None
        if len(self.layers):
            first = self.layers[0]
            new_box = None
            try:
                new_box = first.envelope().forward(first.proj_obj,self.proj_obj)
            except RuntimeError:
                # try clipping layer extent to map
                new_box = self.envelope().forward(self.proj_obj,first.proj_obj)
                new_box.clip(first.envelope())
            for layer in self.layers:
                layer_box = layer.envelope()
                box = None
                try:
                    box = layer_box.forward(layer.proj_obj,self.proj_obj)
                except RuntimeError:
                    # try clipping layer extent to map
                    box = self.envelope().forward(self.proj_obj,first.proj_obj)
                    box.clip(layer_box)
                new_box.expand_to_include(box)
        return new_box
    
    def zoom_to_layers(self,layers):
        first = self.find_layer(layers[0])
        new_box = first.envelope().forward(first.proj_obj,self.proj_obj)
        for lyr in layers:
            layer = self.find_layer(lyr)
            layer_box = layer.envelope()
            box = layer_box.forward(layer.proj_obj,self.proj_obj)
            new_box.expand_to_include(box)
        self.zoom_to_box(new_box)

    def zoom_to_level(self,level):
        c = self.layers_bounds().center()
        self.set_center_and_zoom(c.x,c.y,level=level,geographic=self.proj_obj.geographic)
    
    def max_resolution(self):
        #self.zoom_max()
        map_w,map_h = self.envelope().width(),self.envelope().height()
        return max(map_w / self.width, map_h / self.height)

    def get_scales(self,number):
        max_res = self.max_resolution()
        return [max_res / 2 ** i for i in range(int(number))]        

    def get_scale_for_zoom_level(self,level):
        return self.get_scales(level+1)[level]
    
    # http://trac.mapnik.org/browser/trunk/src/map.cpp#L245
    def set_center_and_zoom(self,lon,lat,level=0,geographic=True):
        coords = mapnik.Coord(lon,lat)
        if geographic and not self.proj_obj.geographic:
            wgs_84 = mapnik.Projection('+init=epsg:4326')
            coords = coords.forward(wgs_84,self.proj_obj)
        w,h = self.width, self.height
        res = self.get_scale_for_zoom_level(level) 
        box = mapnik.Box2d(coords.x - 0.5 * w * res,
                    coords.y - 0.5 * h * res, 
                    coords.x + 0.5 * w * res, 
                    coords.y + 0.5 * h * res)
        self.zoom_to_box(box) 

    def set_center_and_radius(self,lon,lat,radius=None,geographic=True):
        coords = mapnik.Coord(lon,lat)
        box = mapnik.Box2d(coords.x - radius,
                      coords.y - radius,
                      coords.x + radius,
                      coords.y + radius)
        if geographic and not self.proj_obj.geographic:
            wgs_84 = mapnik.Projection('+init=epsg:4326')
            box = box.forward(wgs_84,self.proj_obj)
        self.zoom_to_box(box)

    def zoom_max(self):
        max_extent = mapnik.Box2d(-179.99999694572804,-85.0511285163245,179.99999694572804,85.0511287798066)
        if not self.proj_obj.geographic:
            wgs_84 = mapnik.Projection('+init=epsg:4326')
            max_extent = max_extent.forward(wgs_84,self.proj_obj)
        self.zoom_to_box(max_extent)

    def activate_layers(self,names):
        self.select_layers(names,remove=False)

    def select_layers(self,names,remove=True):
        disactivated = []
        selected = []
        if not isinstance(names,list):
            names = [names]
        for lyr in self.layers:
            if not lyr.name in names and remove:
                lyr.active = False
                disactivated.append(lyr.name)
            else:
                lyr.active = True
                selected.append(lyr.name)
        return selected, disactivated 
    
    def intersecting_layers(self):
        lyrs = []
        for layer in self.layers:
            layer_box = None
            try:
                layer_box = layer.envelope().forward(layer.proj_obj,self.proj_obj)
            except RuntimeError:
                # try clipping layer extent to map
                layer_box = self.envelope().forward(self.proj_obj,layer.proj_obj)
                layer_box.clip(layer.envelope())
            if layer_box.intersects(self.envelope()):
                #layer.active_rules = layer.active_rules(self)
                lyrs.append(layer)
        return lyrs

    def to_wld(self, x_rotation=0.0, y_rotation=0.0):
        """
        Outputs an ESRI world file that can be used to load the resulting
        image as a georeferenced raster in a variety of gis viewers.
        
        '.wld' is the most common extension used, but format-specific extensions
        are also looked for by some software, such as '.tfw' for tiff and '.pgw' for png
        
        A world file file is a plain ASCII text file consisting of six values separated
        by newlines. The format is: 
            pixel X size
            rotation about the Y axis (usually 0.0)
            rotation about the X axis (usually 0.0)
            pixel Y size (negative when using North-Up data)
            X coordinate of upper left pixel center
            Y coordinate of upper left pixel center
         
        Info from: http://gdal.osgeo.org/frmt_various.html#WLD
        """
        extent = self.envelope()
        pixel_x_size = (extent.maxx - extent.minx)/self.width
        pixel_y_size = (extent.maxy - extent.miny)/self.height
        upper_left_x_center = extent.minx + 0.5 * pixel_x_size + 0.5 * x_rotation
        upper_left_y_center = extent.maxy + 0.5 * (pixel_y_size*-1) + 0.5 * y_rotation
        # http://trac.osgeo.org/gdal/browser/trunk/gdal/gcore/gdal_misc.cpp#L1296
        wld_string = '''%.10f\n%.10f\n%.10f\n-%.10f\n%.10f\n%.10f\n''' % (
                pixel_x_size, # geotransform[1] - width of pixel
                y_rotation, # geotransform[4] - rotational coefficient, zero for north up images.
                x_rotation, # geotransform[2] - rotational coefficient, zero for north up images.
                pixel_y_size, # geotransform[5] - height of pixel (but negative)
                upper_left_x_center, # geotransform[0] - x offset to center of top left pixel
                upper_left_y_center # geotransform[3] - y offset to center of top left pixel.
            )
        return wld_string
                  
class _Layer(mapnik.Layer,_injector):

    @property
    def proj_obj(self):
        return EasyProjection(self.srs)

    def set_srs_by_srid(self,srid):
        self.srs = EasyProjection(srid).params()
    
    def active_rules(self,map):
        rules = []
        for style in self.styles:
            sty_obj = map.find_style(style)
            for rule in sty_obj.rules:
                if rule.active(map.scale_denominator()):
                    rules.append({'name':rule.name,'parent':style,'filter':str(rule.filter)})
        return rules


class _Coord(mapnik.Coord,_injector):
    def forward(self,from_prj,to_prj):
        trans = mapnik.ProjTransform(from_prj,to_prj)
        return trans.forward(self)

class _Box2d(mapnik.Box2d,_injector):
    def forward(self,from_prj,to_prj):
        trans = mapnik.ProjTransform(from_prj,to_prj)
        return trans.forward(self)

if __name__ == '__main__':
    import doctest
    doctest.testmod()


import os
import sys
import timeit

from mapnik_utils.version_adapter import Mapnik

mapnik = Mapnik()


class Load(object):
    def __init__(self,mapfile,variables={},output_dir=None,from_string=False,verbose=False):
        self.mapfile = mapfile
        self.variables = variables
        self.from_string = from_string
        self.verbose = verbose
        if output_dir:
            self.output_dir = output_dir
        else:
            if self.mapfile.startswith('http'):
                self.output_dir = os.getcwd() #os.path.expanduser('~/.cascadenik')
            else:
                self.output_dir = os.path.dirname(self.mapfile)
        self.start_time = 0
        self.load_map_time = 0
        
        self.mapfile_types = {'xml':'XML mapfile','mml':'Mapnik Markup Language', 'py':'Python map variable'}
        if self.from_string:
            self.file_type = 'xml'
        else:
            self.file_type = self.get_type()
            self.validate()

    def timer(self):
        self.start_time = timeit.time.time()
    
    def stop(self):
        self.load_map_time = timeit.time.time() - self.start_time
        
    def validate(self):
        if not os.path.exists(self.mapfile) and not self.mapfile.startswith('http'):
            raise AttributeError('Map "%s" not found!' % os.path.abspath(self.mapfile))
        if not self.file_type in self.mapfile_types:
            raise AttributeError('Invalid mapfile type: only these extension allowed: %s' % ', '.join(self.mapfile_types.keys()))
        return True

    def get_type(self):
        if self.mapfile.endswith('xml'):
            return 'xml'
        elif self.mapfile.endswith('mml'):
            return 'mml'
        elif self.mapfile.endswith('py'):
            return 'py'
        else:
            raise ValueError("Unknown Mapfile type: '%s'" % self.mapfile)

    def variable_replace(self):
        import tempfile
        if self.from_string:
            mapfile_string = self.mapfile
        else:
            mapfile_string = open(self.mapfile).read()
        for line in mapfile_string.splitlines():
            for key,value in self.variables.items():
                line.replace(key,value)
        tmp = tempfile.NamedTemporaryFile(suffix='.xml', mode = 'w')
        tmp.write(mapfile_string)
        tmp.flush()
        return tmp.name

    def load_xml(self,m):
        if self.from_string:
            return mapnik.load_map_from_string(m,self.mapfile)
        else:
            return mapnik.load_map(m,self.mapfile)

    def load_mml(self,m):
        import cascadenik
        if hasattr(cascadenik,'VERSION'):
            major = int(cascadenik.VERSION.split('.')[0])
            if major < 1:
                from cascadenik import compile as _compile
                compiled = '%s_compiled.xml' % os.path.splitext(self.mapfile)[0]
                open(compiled, 'w').write(_compile(self.mapfile))
                mapnik.load_map(m, compiled)
            elif major == 1:
                cascadenik.load_map(m,self.mapfile,self.output_dir,verbose=self.verbose)
            elif major > 1:
                raise NotImplementedError('This nik2img version does not yet support Cascadenik > 1.x, please upgrade nik2img to the latest release')
        else:
            from cascadenik import compile as _compile
            compiled = os.path.join(self.output_dir,'%s_compiled.xml' % os.path.splitext(os.path.basename(self.mapfile))[0])
            output = _compile(self.mapfile)
            open(compiled, 'w').write(output)
            mapnik.load_map(m, compiled)

    def load_py(self,m,map_variable='m'):
        """
        Instanciate a Mapnik Map object from an external python script.
        """
        py_path = os.path.abspath(self.mapfile)
        sys.path.append(os.path.dirname(py_path))
        py_module = os.path.basename(py_path).replace('.py','')
        module = __import__(py_module)
        py_map = getattr(module,map_variable,None)
        if not py_map:
            raise ValueError('No variable found in python file with the name: "%s"' % map_variable)
        py_map.width = m.width
        py_map.height = m.height
        return py_map
          
    def load_mapfile(self,m):
        if self.variables:
            self.mapfile = self.variable_replace()
        load = getattr(self,'load_%s' % self.file_type)
        self.timer()
        load(m)
        self.stop()

    def build_map(self,width,height):
        m = mapnik.Map(width,height)
        if self.file_type == 'py':
            return self.load_py(m)
        else:
            self.load_mapfile(m)
            return m


import os
from mapnik_utils.version_adapter import Mapnik

mapnik = Mapnik()

valid_extensions = ['.ttf','.otf','.ttc','.pfa','.pfb','.ttc','.dfont']

class FontHandler(object):
    def __init__(self):
        self.added = []
        self.failed = []
        self.fontdir = mapnik.fontscollectionpath
    
    @property
    def available(self):
        return [f for f in mapnik.FontEngine.face_names()]

    def add_fonts(self,fonts):
        engine = mapnik.FontEngine.instance()
        for font in fonts:
            if os.path.isdir(font):
                found = False;
                for dirpath, _, filenames in os.walk(font):
                    for filename in filenames:
                        if os.path.splitext(filename)[1] in valid_extensions:
                            font_name = os.path.join(dirpath, filename)
                            if engine.register_font(font_name):
                                self.added.append(font_name)
                            else:
                                self.failed.append(font_name)
            else:
                if engine.register_font(font):
                    self.added.append(font)
                else:
                    self.failed.append(font)




from mapnik_utils.version_adapter import Mapnik

mapnik = Mapnik()

TRANSFORM_ERROR = 'Only Mapnik >= 0.6.0 supports projected transforms, see: http://trac.mapnik.org/ticket/117'

class ProjTransform(object):
    def __init__(self,from_prj,to_prj):
        self.to_prj = to_prj
        self.from_prj = from_prj
    
    def forward(self,geom):
        if self.to_prj.geographic and self.from_prj.geographic:
            return geom
        elif not self.to_prj.geographic and self.from_prj.geographic:
            return mapnik.forward_(geom,self.to_prj)
        elif self.to_prj.geographic and not self.from_prj.geographic:
            return mapnik.inverse_(geom,self.to_prj)
        elif not self.to_prj.geographic and not self.from_prj.geographic:
            if self.to_prj.params() == self.from_prj.params():
                return geom
            else:
                raise NotImplementedError(TRANSFORM_ERROR)
        
    def backward(self,geom):
        if self.to_prj.geographic and self.from_prj.geographic:
            return geom
        elif not self.to_prj.geographic and self.from_prj.geographic:
            return mapnik.inverse_(geom,self.to_prj)
        elif self.to_prj.geographic and not self.from_prj.geographic:
            return mapnik.forward_(geom,self.to_prj)
        elif not self.to_prj.geographic and not self.from_prj.geographic:
            if self.to_prj.params() == self.from_prj.params():
                return geom
            else:
                raise NotImplementedError(TRANSFORM_ERROR)

BoostPythonMetaclass = mapnik.Coord.__class__
                
class _injector(object):
    class __metaclass__(BoostPythonMetaclass):
        def __init__(self, name, bases, dict):
            for b in bases:
                if type(b) not in (self, type):
                    for k,v in dict.items():
                        setattr(b,k,v)
            return type.__init__(self, name, bases, dict)

class _Map(mapnik.Map,_injector):

    def scale_denominator(self):
        srs = mapnik.Projection(self.srs)
        return mapnik.scale_denominator(self,srs.geographic)
    
    def resize(self,w,h):
        self.width = w
        self.height = h

class _Coord(mapnik.Coord,_injector):
    def forward(self,obj):
        return mapnik.forward_(self,obj)
    def inverse(self,obj):
        return mapnik.inverse_(self,obj)

class _Envelope(mapnik.Envelope,_injector):
    def forward(self,obj):
        return mapnik.forward_(self,obj)
    def inverse(self,obj):
        return mapnik.inverse_(self,obj)


#!/usr/bin/env python

__version__ = '0.8.0'
__author__ = 'Dane Springmeyer (dane [ -a- ] dbsgeo.com)'
__copyright__ = 'Copyright 2010, Dane Springmeyer'
__license__ = 'BSD'

import os
import sys
import optparse
import tempfile

parser = optparse.OptionParser(usage="""%prog <mapfile> <image> [options]

Example usage
-------------

Full help:
 $ %prog -h (or --help for possible options)

Read XML, output image:
 $ %prog mapfile.xml image.png

Read MML in verbose mode
 $ %prog mapfile.mml image.png -v

Read MML, pipe to image
 $ %prog mapfile.mml - > image.png

Accept piped XML
$ <xml stream> | %prog - image.png

""", version='%prog ' + __version__)

def make_float_list(option, opt, value, parser):
    try:
        values = [float(i) for i in value.split(',')]
    except:
        parser.error("option %s: invalid float values: '%s'" % (opt,value))
    setattr(parser.values, option.dest, values)

def make_int_list(option, opt, value, parser):
    try:
        values = [int(i) for i in value.split(',')]
    except:
        parser.error("option %s: invalid integer values: '%s'" % (opt,value))
    setattr(parser.values, option.dest, values)

def make_list(option, opt, value, parser):
    values = [i.strip() for i in value.split(',')]
    setattr(parser.values, option.dest, values)
    

parser.add_option('-f', '--format', dest='format',
                  help='Format of image: png (32 bit), png256 (8 bit), jpeg, pdf, svg, ps, ARGB32, RGB24, or all (will loop through all formats).')

parser.add_option('-c', '--center', dest='center', nargs=2,
                  help='Center coordinates. A long,lat pair e.g. -122.3 47.6 (Seattle)',
                  type='float',
                  action='store')

parser.add_option('-z', '--zoom', dest='zoom',
                  help='Zoom level',
                  type='int',
                  action='store')

parser.add_option('-b','--bbox', dest='bbox',
                  type='float', nargs=4,
                  help='Geographical bounding box. Two long,lat pairs e.g. -126 24 -66 49 (United States)',
                  action='store')

parser.add_option('-e', '--projected-extent', dest='extent', nargs=4,
                  help='Projected envelope/extent. Two coordinate pairs in the projection of the map',
                  type='float',
                  action='store')
                  
parser.add_option('-r', '--radius', dest='radius',
                  help='Zoom to radius (in map units) around center',
                  type='float',
                  action='store')

parser.add_option('--zoom-to-layers', dest='zoom_to_layers',
                  help='Zoom to combined extent of one ore more listed layers by name (comma separated)',
                  type='string', # actually results in a comma-delimited list
                  action='callback',
                  callback=make_list)

parser.add_option('-m', '--max-extent', dest='max_extent', nargs=4,
                  help='Projected envelope/extent. Two coordinate pairs in the projection of the map',
                  type='float',
                  action='store')

parser.add_option('--bbox-factor', dest='bbox_factor',
                  type='float',
                  help='Expand or contract final map bounds by factor (positive values will multiple bbox, negative values will divide bbox',
                  action='store')
                                    
parser.add_option('-s', '--srs',
                  dest='srs',
                  help='Spatial reference system to project the image into - accepts either <epsg:code>, <proj4 literal>, or a url like http://spatialreference.org/ref/sr-org/6')

parser.add_option('-d', '--dimensions', dest='dimensions', nargs=2,
                  help='Pixel dimensions of image (width,height)',
                  type='int',
                  default = (600,400),
                  action='store')

parser.add_option('-l', '--layers',
                  type='string', # actually results in a comma-delimited list
                  help='List of layers by name to render (comma separated)',
                  action='callback',
                  callback=make_list)

parser.add_option('-n', '--dry-run', dest='dry_run',
                  help='Construct map but do not render output',
                  action='store_true')

parser.add_option('-w','--world-file',
                  help="Georeference the image by providing a file extention for worldfile output ( ie 'wld')")

parser.add_option('-x', '--xml', dest='save_xml',
                  help='Serialize the map to xml.')

parser.add_option('-a', '--app', dest='app',
                  help='Application to open the resulting image.')

parser.add_option('--profile', dest='profile',
                  action='store_true', default=False,
                  help='Output a cProfile report')

parser.add_option('-v', '--verbose', dest='verbose',
                  help='Make a bunch of noise',
                  action='store_true')
                  
parser.add_option('-p', '--pause', dest='pause',
                  help='Seconds to pause after each step', type='int',
                  action='store')

parser.add_option('-t', '--trace-steps', dest='trace_steps',
                  type='string', # actually results in a comma-delimited list
                  help='Step(s) at which to set a python debugger trace (separated by commas)',
                  action='callback',
                  callback=make_int_list)

parser.add_option('--no-color', dest='no_color',
                  action='store_true', default=False,
                  help='Turn off colored terminal output')

parser.add_option('--no-open', dest='no_open',
                  action='store_true', default=False,
                  help='Skip opening of image in default viewer')
                  
parser.add_option('--fonts',
                  type='string', # actually results in a comma-delimited list
                  help='List of paths to .ttf or .otf fonts to register (comma separated)',
                  action='callback',
                  callback=make_list)

parser.add_option('--mapnik-version', dest='mapnik_version',
                  action='store', default=None,
                  help='Use the mapnik2 python bindings if they exist', type='int')

parser.add_option('--zip', dest='zip_compress',
                  action='store_true', default=False,
                  help='Apply zip compression to output image (and other files by same name)')

parser.add_option('--scale-factor', dest='scale_factor',
                  type='float',
                  help='Pass a scale factor to the Mapnik renderer (experimental)',
                  action='store')

parser.add_option('--aspect-fix-mode', dest='aspect_fix_mode',
                  type='string',
                  help='Pass an aspect_fix_mode string (case-insensitive match of http://media.mapnik.org/api_docs/python/mapnik._mapnik.aspect_fix_mode-class.html)',
                  action='store')
                      
if __name__ == '__main__':
    (options, args) = parser.parse_args()
    
    # we're actually starting now so pull in mapnik
    # this version adapter is a nasty hack to globally
    # control usage of either mapnik or mapnik2
    from mapnik_utils.version_adapter import Mapnik
    mapnik = Mapnik(options.mapnik_version)
    
    if options.scale_factor:
        if not mapnik.mapnik_version() >= 800:
            parser.error('\nPassing a scale factor requires at least mapnik >= 0.8.0 (aka Mapnik2)\n')
    
    if len(args) == 0:
        parser.error('\nPlease provide the path to a Mapnik xml or Cascadenik mml file\n')
    
    elif len(args) == 1 and not options.dry_run:
        parser.error('\nPlease provide an output image name\n')

    if args[0] == '-' and not sys.stdin.isatty():
        xml = sys.stdin.read()
        if hasattr(mapnik,'load_map_from_string'):
            options.from_string = True
            mapfile = xml
        else:
            options.from_string = False
            (handle, mapfile) = tempfile.mkstemp('.xml', 'mapfile_string')
            os.close(handle)
            open(mapfile, 'w').write(xml)
    else:
        mapfile = args[0]

    if len(args) > 1:
        options.image = args[1]
        #if options.image == '-':
        #    options.pipe_output = True
        #else:
        #    options.pipe_output = False
           
    options.width, options.height = options.dimensions
    if not options.format and hasattr(options,'image'):
        if options.image == '-':
            options.format = 'png'
        elif not options.image.endswith('png'):
            try:
                options.format = options.image.split('.')[-1]
            except:
                pass

    def main():
        from mapnik_utils.composer import ComposeDebug
        nik_map = ComposeDebug(mapfile,**options.__dict__)
        if options.no_open:
            nik_map.render()
        else:
            nik_map.open()
    
    if options.profile:
        import cProfile
        cProfile.run('main()', sort=1)
    else:
        main()



import os
from subprocess import Popen, PIPE

def call(cmd,fail=False):
    try:
        response = Popen(cmd.split(' '),stdin=PIPE, stdout=PIPE, stderr=PIPE)
        cm = response.communicate()
        return cm[0]
    except Exception, e:
        if fail:
            raise SystemExit(e)
        else:
            return None

def color_print(color, text, no_color=False):
    """
    Accepts an integer key for one of several color choices along with the text string to color
      keys = 1:red, 2:green, 3:yellow, 4: dark blue, 5:pink, 6:teal blue, 7:white
    Prints a colored string of text.
    """
    if not os.name == 'nt' and not no_color:
        print "\033[9%sm%s\033[0m" % (color,text)
    else:
        print text

def color_text(color, text, no_color=False):
    """
    Accepts an integer key for one of several color choices along with the text string to color
      keys = 1:red, 2:green, 3:yellow, 4: dark blue, 5:pink, 6:teal blue, 7:white
    Returns a colored string of text.
    """
    if not os.name == 'nt' and not no_color:
        return "\033[9%sm%s\033[0m" % (color,text)
    else:
        return text


import re
import sys
import urllib
import socket
    
socket.setdefaulttimeout(6)

from mapnik_utils.version_adapter import Mapnik

mapnik = Mapnik()

MERC_PROJ4 = '+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +no_defs +over'

class EasyProjection(mapnik.Projection):
    def __init__(self,srs):
        self.srs = srs
        self.srid = None
        self.method, self.proj = self.get_proj(self.srs)
        try:
            mapnik.Projection.__init__(self,self.proj)
        except RuntimeError, E:
            proj = None
            if self.method == 'epsg srid':
                sys.stderr.write('SRS not found locally checking http://spatialreference.org...\n')
                sr_org = 'http://spatialreference.org/ref'
                srs_types = ['epsg','esri','sr-org','iau2000']
                for provider in srs_types:
                    url = '%s/%s/%s/' % (sr_org,provider, self.srs)      
                    sys.stderr.write('Checking... %s\n' % url)
                    proj = self.get_from_sr_org(url)
                    if proj:
                        break
            if proj:
                self.srid = self.srs
                mapnik.Projection.__init__(self,proj)
            else:
                raise RuntimeError('Sorry, that projection was not found.\nMapnik error: "%s"' % E)
    
    @property
    def proj_obj(self):
        return mapnik.Projection(self.proj)

    def get_from_sr_org(self,srs):
        # http://trac.osgeo.org/gdal/changeset/11772
        url = '%s/proj4/' % srs.strip('/')
        resp = urllib.urlopen(url).read()
        if len(resp) > 1000:
            return None
        else:
            sys.stderr.write('Found... %s\n' % url)
        return resp
            
    def get_proj(self,srs):
        if isinstance(srs,(str,unicode)):
            if srs.isdigit():
                srs = int(srs)
        if isinstance(srs,int):
            self.srid = self.srs
            if srs == 900913 or srs == 3785:
                return 'custom proj4 stored in nik2img', MERC_PROJ4
            else:
                return 'epsg srid', '+init=epsg:%s' % srs
        elif re.match('^\+proj=.+$', srs):
            return 'proj4 literal', srs
        elif srs.startswith('http://'):
            proj4 = self.get_from_sr_org(srs)
            if not proj4:
                raise RuntimeError('%s failed to return a proj4 value' % srs)
            return 'proj4 from http://spatialreference.org', proj4
        elif re.match('^\+init=epsg:\d+$', srs.lower()):
            self.srid = self.srs
            return 'epsg srid', srs
        elif re.match('^epsg:\d+$', srs.lower()):
            self.srid = self.srs
            if srs == "epsg:900913" or srs == "epsg:3785":
                return 'custom proj4 stored in nik2img', MERC_PROJ4
            else:
                return 'epsg srid','+init=%s' % srs
        else:
            raise RuntimeError('Invalid value for initializing Projection: %s\n Accepts epsg code, proj4 string, or spatialreference.org url...' % srs)


from mapnik_utils.version_adapter import Mapnik

mapnik = Mapnik()

import os
import sys
import platform
from timeit import time
from pdb import set_trace

# mapnik_utils
from subprocess import Popen
from mapnik_utils.renderer import Render
from mapnik_utils.loader import Load
from mapnik_utils.tools import call, color_text, color_print


class Compose(object):
    """
    """
    def __init__(self,mapfile,**kwargs):
        """
        """
        self.mapfile = mapfile

        self.image = None
        self.format = 'png'
        self.width = 600
        self.height = 400
        self.bbox = None
        self.zoom = None
        self.center = None
        self.radius = None
        self.zoom_to_layers = None
        self.extent = None
        self.max_extent = None
        self.bbox_factor = None
        self.srs = None
        self.layers = None
        self.re_render_times = None
        self.post_step_pause = None
        self.max_resolution = None
        self.find_and_replace = None
        self.world_file = None
        self.fonts = []
        self.save_xml = None
        self.app = None
        self.dry_run = False
        self.from_string = False
        self.zip_compress = False
        self.changed = []
        self.font_handler = None
        self.map = None
        self.rendered = False
        self.verbose = False
        self.scale_factor = None
        self.aspect_fix_mode = None
        
        self.start_time = 0
        self.load_map_time = 0
        
        if kwargs:
            self.handle_options(kwargs)

        if not self.verbose:
            self.msg = self.quiet
        self.prepare()
        self.setup()
        self.build()

    def handle_options(self,options):
        for opt in options.items():
            if opt[1] is not None and hasattr(self,opt[0]):
                setattr(self,opt[0],opt[1])
                self.changed.append(opt[0])

    def prepare(self):
        self.format = self.format.replace('image/','')
        self.mime = 'image/%s' % self.format.replace('256','')
        if self.fonts:
            self.register_fonts()

    def setup(self):
        pass

    def output_error(self, msg, E=None):
        if E:
            msg += E
        raise sys.exit(msg)

    def msg(self, msg, warn=False):
        sys.stderr.write('%s\n' % msg)

    def quiet(self, msg):
        pass

    def register_fonts(self):
        self.msg('Registering fonts...')
        from fonts import FontHandler
        self.font_handler = FontHandler()
        self.font_handler.add_fonts(self.fonts)
        if self.font_handler.failed:
            self.msg("Failed to register: '%s'" % self.font_handler.failed)

    def build(self):
        self.msg('Loading mapfile...')
        
        builder = Load(self.mapfile,variables={},from_string=self.from_string,verbose=self.verbose)
        if not self.from_string:
            self.msg('Loaded %s...' % self.mapfile)
        else:
            self.msg('Loaded XML from string')
        self.map = builder.build_map(self.width,self.height)

        if self.srs:
            self.msg('Setting srs to: %s' % self.srs)
            self.map.set_easy_srs(self.srs)

        if self.layers:
            selected, disactivated = self.map.select_layers(self.layers)
            self.msg('Selected layers: %s' % selected)
            if not selected:
                self.output_error('Layer not found: available layers are: "%s"' % ',  '.join(disactivated))
                
        # set up behavior for fixing relationship between map aspect and bbox aspect
        if self.aspect_fix_mode:
            if not hasattr(self.map, 'aspect_fix_mode'):
                self.output_error('Your version of Mapnik does not support setting the aspect_fix_mode (only available in >= Mapnik 0.6.0')
            try:
                mode = getattr(mapnik.aspect_fix_mode,self.aspect_fix_mode.upper())
            except AttributeError:
                self.output_error('Error setting aspect_fix_mode, accepted values are: %s' % ', '.join(mapnik.aspect_fix_mode.names.keys()))
            
            self.map.aspect_fix_mode = mode

        # zoom to max extent at beginning if we later need to 
        # zoom to a center point
        # or need to zoom to a zoom-level
        self.msg('Setting Map view...')
        
        if self.center or not self.zoom is None:
            if self.max_extent:
                self.msg('Zooming to max extent: %s' % self.max_extent) 
                self.map.zoom_to_box(mapnik.Box2d(*self.max_extent))
            else:
                self.map.zoom_max()
                self.msg('Zoomed to *estimated* max extent: %s' % self.map.envelope()) 

        if self.center and not self.zoom is None:
            self.msg('Zooming to Center (%s) and Zoom Level "%s"' % (self.center,self.zoom))
            self.map.set_center_and_zoom(self.center[0],self.center[1],self.zoom)
        elif self.center and self.radius:
            self.msg('Zooming to Center (%s) and Radius "%s"' % (self.center,self.radius))
            self.map.set_center_and_radius(self.center[0],self.center[1],self.radius)
        elif not self.zoom is None:
            self.msg('Zooming to Zoom Level "%s"' % (self.zoom))
            self.map.zoom_to_level(self.zoom)
        elif self.zoom_to_layers:
            self.msg('Zooming to Layers: "%s"' % (self.zoom_to_layers))
            self.map.activate_layers(self.zoom_to_layers)
            if len(self.zoom_to_layers) > 1:
                self.map.zoom_to_layers(self.zoom_to_layers)
            else:
                self.map.zoom_to_layer(self.zoom_to_layers[0])
        else:
            if self.extent:
                env = mapnik.Box2d(*self.extent)
                self.msg('Zooming to custom projected extent: "%s"' % env)
                self.map.zoom_to_box(env)
                from_prj = mapnik.Projection(self.map.srs)
                to_prj = mapnik.Projection('+proj=latlong +datum=WGS84')
                bbox = env.forward(from_prj,to_prj)
                self.msg('Custom extent in geographic coordinates: "%s"' % bbox)
            elif self.bbox:
                env = mapnik.Box2d(*self.bbox)
                self.msg('Zooming to custom geographic extent: "%s"' % env)
                from_prj = mapnik.Projection('+proj=latlong +datum=WGS84')
                to_prj = mapnik.Projection(self.map.srs)
                self.map.zoom_to_box(env.forward(from_prj,to_prj))
            else:
                self.map.zoom_all()
                self.msg('Zoom to extent of all layers: "%s"' % self.map.envelope())
        
        if self.bbox_factor:
            if self.bbox_factor > 0:
                bbox = self.map.envelope() * self.bbox_factor
            else:
                bbox = self.map.envelope() / self.bbox_factor
            self.map.zoom_to_box(bbox)
            self.msg('Adjusting final extent by factor of %s: "%s"' % (self.bbox_factor,self.map.envelope()))
            
        self.msg('Finished setting extents...')
        
        if self.save_xml:
            mapnik.save_map(self.map,self.save_xml)
        
        return builder

    def render(self):
        if not self.map:
            self.build()
        
        if self.dry_run:
            self.output_error("Dry run completed successfully...")

        renderer = Render(self.map,self.image,self.format,self.world_file,self.zip_compress,self.scale_factor)
        if self.image == '-':
            renderer.print_stream()
        else:
            renderer.render_file()
        self.rendered = True
        return renderer
    
    def open(self, app=None):
        """
        Routine to open the rendered image or folder of images from the filesystem.
        """
        self.render()
        if not app and self.app:
            app = self.app
        if os.name == 'nt':
            if app:
                self.msg('Overriding default image viewer not supported on Win32')
            call('start %s' % self.image.replace('/','\\'))
        elif platform.uname()[0] == 'Linux':
            if app:
                call('%s %s' % (app,self.image))
            else:
                # make blind and dumb attempt to open images, but don't block while open
                try:
                    cmd = 'xdg-open %s' % self.image
                    Popen(cmd.split(' '))
                except OSError:
                    try:
                        cmd = 'gthumb %s' % self.image
                        Popen(cmd.split(' '))
                    except OSError:
                        try:
                            cmd = 'display %s' % self.image
                            Popen(cmd.split(' '))
                        except OSError:
                            pass
        elif platform.uname()[0] == 'Darwin':
            if app:
                call('open %s -a %s' % (self.image, app))
            else:
                call('open %s' % self.image)


class ComposeDebug(Compose):
    """
    """
    def __init__(self,mapfile,**kwargs):
        self.no_color = kwargs.get('no_color')   
        self.pause = kwargs.get('pause')
        self.trace_steps = kwargs.get('trace_steps')
        self.verbose = kwargs.get('verbose')

        self.timing_started = False
        self.step_counter = 1
        self.start_time = 0

        Compose.__init__(self,mapfile,**kwargs)

    def setup(self):
        if self.trace_steps and not self.verbose:
            self.verbose = True
            self.debug_msg('PDB trace requested, automatically entering verbose mode')

    def prepare(self):
        super(ComposeDebug,self).prepare()
        self.timing_started = True
        self.start_time = time.time()
        self.debug_msg('Nik2img starting...')
        self.debug_msg('Format: %s' % self.format)
        #self.debug_msg('mime: %s' % self.mime)

    def build(self):
        #try:
        builder = super(ComposeDebug,self).build()
        #except Exception, E:
        #    self.output_error(E)        
        self.last_step('Loading map took... ', builder.load_map_time)
        if self.verbose:
            self.debug_msg('SRS: %s' % self.map.srs)
            if self.map.proj_obj.srid:
                self.debug_msg('SRID: %s' % self.map.proj_obj.srid)
            self.debug_msg('Map extent: %s' % self.map.envelope())
            self.debug_msg('Map long/lat bbox: %s' % self.map.lon_lat_bbox())
            self.debug_msg('Map center: %s' % self.map.envelope().center())
            self.debug_msg('Map long/lat center: %s' % self.map.lon_lat_bbox().center())
            self.debug_msg('Map scale denominator: %s' % self.map.scale_denominator())
            if self.layers:
                self.debug_msg('Active layers: %s' % self.map.active_layers())
            if self.map.layers_bounds():
                self.debug_msg('Extent of all layers: %s' % self.map.layers_bounds())
                self.debug_msg('Long/lat extent of all layers: %s' % self.map.lon_lat_layers_bounds())
                self.debug_msg('Long/lat center of all layers: %s' % self.map.lon_lat_layers_bounds().center())
    
            lyrs = self.map.intersecting_layers()
            if not len(lyrs):
                self.debug_msg("No layers intersecting map!",warn=True)
            else:
                self.debug_msg("Layers intersecting map: [%s]" % ', '.join([l.name for l in lyrs]))
            self.debug_msg("At current scale of '%s'..." % self.map.scale())
            for lyr in lyrs:
                active_rules = lyr.active_rules(self.map)
                if len(active_rules):
                    active_styles = []
                    for ar in active_rules:
                        rsty = ar['parent']
                        if rsty not in active_styles:
                            active_styles.append(rsty)
                    active_styles_string = ', '.join(active_styles) 
                    self.debug_msg("Layer '%s' has %s active rule(s) in styles: %s" % (lyr.name,len(active_rules),active_styles))
                    active_filter_strings = ', '.join(["'%s'" % s['name'] or s['filter'] for s in active_rules]) 
                    if 'true' not in active_filter_strings:
                        print 'Active rules: %s' % (active_filter_strings)
                else:
                    self.debug_msg("Layer '%s' is NOT visible" % lyr.name,warn=True)
        
    def render(self):
        if not self.map:
            self.debug_msg('Calling build from render...')
        self.debug_msg('Starting rendering...')            
        #try:
        renderer = super(ComposeDebug,self).render()
        #except Exception, E:
        #    self.output_error(E)
        self.last_step('Rendering image took... ', renderer.render_time)
        self.debug_msg('Finished rendering map to... %s' % self.image)
        self.total_time()

    def register_fonts(self):
        super(ComposeDebug,self).register_fonts()
        if len(self.font_handler.added):
            self.debug_msg("Registered: '%s'" % self.font_handler.added)
        elif len(self.font_handler.failed):
            self.debug_msg("Available fonts are: '%s'" % self.font_handler.available,warn=True)
      
    def output_error(self, msg, E=None):
        if E:
            sys.stderr.write(color_text(1, '// --> %s: \n\t %s\n' % (msg, E),self.no_color))
        else:
            sys.stderr.write(color_text(1, '// --> %s \n' % msg,self.no_color))
        sys.exit(1)

    def msg(self, msg, warn=False):
        self.debug_msg(msg,warn=warn)

    def debug_msg(self, msg, warn=False):
        """
        Output a colored message or warning, incrementing the step_counter
        to enable a pdb trace to be set at any point a verbose message is printed.
        """
        color = 2
        if warn:
            color = 1
        if self.verbose:
            text = 'Step: %s // --> %s\n' % (self.step_counter, msg)
            sys.stderr.write(color_text(color,text,self.no_color))
        if self.pause:
            for second in range(1, (int(self.pause)+1)):
                sys.stderr.write('%s ' % color_text(5,second,self.no_color))
                time.sleep(1)
                sys.stderr.flush()
            sys.stderr.write('... \n')
        if self.trace_steps:
            if self.step_counter in self.trace_steps:
                try:
                    print ">>> Entering PDB interpreter (press 'c' to leave)"
                    set_trace()
                except KeyboardInterrupt:
                    pass
        self.step_counter += 1

    def get_time(self, time):
        """
        Get the time and either seconds or minutes format.
        """
        if time/60 < 1:
            seconds = '%s seconds' % str(time)
            return seconds
        else:
            minutes = '%s minutes' % str(time/60)
            return minutes
    
    def total_time(self, last_step=None):
        if self.verbose:
            total = (time.time() - self.start_time)
            out = 'Total Nik2img run time: %s' % (self.get_time(round(total,4)))
            if last_step:
                out += '| Last step: %s'% self.get_time(round(last_step,8))
            val = color_text(4,out,self.no_color)
            sys.stderr.write('%s\n' % val)

    def last_step(self,msg,timing):
        if self.verbose:
            out = '%s %s' % (msg, self.get_time(round(timing,4)))
            val = color_text(4,out,self.no_color)
            sys.stderr.write('%s\n' % val)


import os
import sys
import timeit
import zipfile
import tempfile
from glob import glob

from mapnik_utils.version_adapter import Mapnik

mapnik = Mapnik()

from mapnik_utils.tools import call

try:
    import cairo
    HAS_CAIRO = True
    cairo_mapping = {'svg':cairo.SVGSurface,
                'pdf':cairo.PDFSurface,
                'ps':cairo.PSSurface,
                'ARGB32':cairo.FORMAT_ARGB32,
                'RGB24':cairo.FORMAT_RGB24
                }

except ImportError:
    HAS_CAIRO = False


# Win32 workaround graciously provided by crschmidt
# from http://svn.tilecache.org/trunk/tilecache/TileCache/Service.py
def binaryPrint(binary_data):
    """This function is designed to work around the fact that Python
       in Windows does not handle binary output correctly. This function
       will set the output to binary, and then write to stdout directly
       rather than using print."""
    try:
        import msvcrt
        msvcrt.setmode(sys.__stdout__.fileno(), os.O_BINARY)
    except:
        pass
    sys.stdout.write(binary_data)


class Render(object):
    def __init__(self,m,image,format,world_file_ext=None,zip_compress=False,scale_factor=None):
        
        self.m = m
        self.image = image
        self.format = format
        self.world_file_ext = world_file_ext
        self.zip_compress = zip_compress
        self.scale_factor = scale_factor
        
        self.start_time = 0
        self.render_time = 0
        
        self.ALL_FORMATS = {}
        self.AGG_FORMATS = {'png':'png','png256':'png','jpeg':'jpg','tif':'tif','tiff':'tif'}
        self.CAIRO_FILE_FORMATS = {'svg':'svg','pdf':'pdf','ps':'ps'}
        self.CAIRO_IMAGE_FORMATS = {'ARGB32':'png','RGB24':'png'}
        self.setup_formats()

    def timer(self):
        self.start_time = timeit.time.time()
    
    def stop(self):
        self.render_time = timeit.time.time() - self.start_time

    def setup_formats(self):
        self.ALL_FORMATS.update(self.AGG_FORMATS)
        self.ALL_FORMATS.update(self.CAIRO_FILE_FORMATS)
        self.ALL_FORMATS.update(self.CAIRO_IMAGE_FORMATS)
    
    def write_wld(self,rendered_image):
        path = os.path.splitext(rendered_image)[0]
        f_ptr = '%s.%s' % (path, self.world_file_ext)
        #sys.stderr.write('Saved world file to.. %s\n' % f_ptr)
        f = open(f_ptr, 'w')
        f.write(self.m.to_wld())
        f.close()
    
    def zip_up(self,rendered_image):
        path = os.path.splitext(rendered_image)[0]
        z_name = '%s.zip' % (path)
        zip_file = zipfile.ZipFile(z_name, 'w', zipfile.ZIP_DEFLATED)
        # all files of that same name (like a world file)
        # of variable ext other than other zip files
        files = glob('%s.*' % path)
        for item in files:
            if not item.endswith('zip'):
                zip_file.write(item, arcname=os.path.basename(item))
        #if readme:
        #    zip.writestr('README.txt',readme)
        zip_file.close()

    
    def stream(self): 
        """
        Routine to render the an image to a string
        """
        self.timer()
        if self.format in self.CAIRO_FILE_FORMATS:
            (handle, cairo_tmp) = tempfile.mkstemp('.%s' % self.format, 'nik2img-cairo_tmp-tmp')
            os.close(handle)
            self.local_render_wrapper(self.m, cairo_tmp, self.format)
            return open(cairo_tmp,'rb').read()
        else:
            im = mapnik.Image(self.m.width,self.m.height)
            if self.scale_factor:
                mapnik.render(self.m,im,self.scale_factor)
            else:
                mapnik.render(self.m,im)
            return im.tostring(self.format)
        self.stop()

    def print_stream(self):
        if sys.platform == 'win32':
            binaryPrint(self.stream())
        else:
            print self.stream() 

    def local_render_wrapper(self,*args):
        """
        Abstraction wrapper for calling for map images rendered with either AGG or Cairo.
        """        
        if args[2] in self.CAIRO_FILE_FORMATS and hasattr(mapnik,'mapnik_version') and mapnik.mapnik_version() < 700:
            self.render_with_pycairo(*args)
        # todo: support rendering to image formats with cairo
        #elif args[2] in self.CAIRO_IMAGE_FORMATS:
            #self.render_with_pycairo(*args)
        else:
            self.render_to_file(*args)

    def render_with_pycairo(self,*args):
        """
        Routine to render the requested Cairo format.
        """
        if not HAS_CAIRO:
            raise ImportError('PyCairo is not installed or available, therefore you cannot write to svg, pdf, ps, or cairo-rendered png')
        else:
            context = [args[1], self.m.width, self.m.height]
            if args[2] in self.CAIRO_FILE_FORMATS:
                surface = cairo_mapping[args[2]](*context)
            elif args[2] in self.CAIRO_IMAGE_FORMATS:
                surface = cairo.ImageSurface(cairo_mapping[args[2]], *context[1:])
            self.timer()
            mapnik.render(args[0],surface)
            if args[2] in self.CAIRO_IMAGE_FORMATS:
                surface.write_to_png(args[1])
            surface.finish()
            self.stop()
            if self.world_file_ext:
                self.write_wld(args[1])
            if self.zip_compress:
                self.zip_up(args[1])

    def call_all_cairo(self, basename):
        """
        Abstraction wrapper to allow for the same call
        to any image and file formats requested from Cairo.
        """
        if not HAS_CAIRO:
            raise ImportError('PyCairo is not installed or available, therefore you cannot write to svg, pdf, ps, or cairo-rendered png')
        else:
            for k, v in self.CAIRO_FILE_FORMATS.iteritems():
                path = '%s_%s.%s' % (basename,k,v)
                self.render_with_pycairo(self.m,path,k)
            for k, v in self.CAIRO_IMAGE_FORMATS.iteritems():
                path = '%s_%s.%s' % (basename,k,v)
                self.render_with_pycairo(self.m,path,k)

    def render_to_file(self,*args):
        """
        Routine to render the requested AGG format.
        """
        format = args[2]
        if format in ('tif','tiff'):
            self.timer()
            (handle, png_tmp) = tempfile.mkstemp('.png', 'nik2img-tmp')
            os.close(handle)
            self.world_file_ext = 'wld'
            self.write_wld(png_tmp)
            im = mapnik.Image(args[0].width,self.m.height)
            if self.scale_factor:
                mapnik.render(args[0],im,self.scale_factor)
            else:
                mapnik.render(args[0],im)
            im.save(png_tmp,'png')
            # todo - figure out more reasonable defaults
            opts = ' -ot Byte -co COMPRESS=JPEG -co JPEG_QUALITY=100'
            base_cmd = 'gdal_translate %s %s -a_srs "%s" %s'
            cmd = base_cmd % (png_tmp,args[1],args[0].srs,opts)
            #print call(cmd,fail=True)
            os.system(cmd)
            self.stop()
        else:
            self.timer()
            if self.scale_factor:
                args = args + (self.scale_factor,)
            mapnik.render_to_file(*args)
            if self.world_file_ext:
                self.write_wld(args[1])
            self.stop()
        if self.zip_compress:
            self.zip_up(args[1])
            
    def call_all_agg(self, basename):
        """
        Abstraction wrapper to allow for calling 
        any requested AGG Formats.
        """
        for k, v in self.AGG_FORMATS.iteritems():
            path = '%s_%s.%s' % (basename,k,v)
            self.render_to_file(self.m,path,k)
           
    def render_file(self): 
        """
        Routine to render the output image(s) for all requested formats and resolutions.
        """
        dirname, basename = os.path.dirname(self.image),os.path.basename(self.image)
        if basename:
            if not True in [self.image.split('.')[-1].lower() == ext for ext in self.ALL_FORMATS]:
                raise AttributeError("Unrecognized format (needs .ext) or directory (needs trailing /).")
        else:
            basename_from_dir = dirname.split('/')[-1]
            if not basename_from_dir: 
                basename_from_dir = 'nik2img_output'
            if dirname == '':
                raise IOError("Must write to either file or directory")
        if not os.path.exists(dirname) and dirname != '':
            try:
                os.mkdir(dirname)
            except OSError:
                os.makedirs(dirname)
        if not dirname.endswith('/'):
            dirname = dirname + '/'
        if self.format == 'all':
            if basename:
                raise IOError("Must write to a directory/ to produce all formats")
            else:                    
                self.call_all_agg(dirname + basename_from_dir)
                if HAS_CAIRO:
                    self.call_all_cairo(dirname + basename_from_dir)
        else:
            if not basename:
                self.local_render_wrapper(self.m, dirname + basename_from_dir + '.' + self.format.rstrip('256'), self.format)
            else:
                self.local_render_wrapper(self.m, self.image, self.format)


from distutils.core import setup

version = '0.8.0'
app = 'nik2img'
description = 'A command line style renderer for the Mapnik C++/Python mapping toolkit'
url = 'http://mapnik-utils.googlecode.com/'
readme = file('README.txt','rb').read()

setup(name='%s' % app,
      version=version,
      description=description,
      long_description=readme,
      author='Dane Springmeyer',
      author_email='dane@dbsgeo.com',
      requires=['Mapnik'],
      keywords='Mapnik,GIS,python,mapping,graphics,geospatial',
      url=url,
      #py_modules=['%s' % app],
      packages=['mapnik_utils'],
      scripts = ['nik2img.py'],
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Console',
          'Environment :: Web Environment',
          'Intended Audience :: End Users/Desktop',
          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Topic :: Scientific/Engineering :: GIS',
          'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
          'Topic :: Utilities'],
      )



class Mapnik:
    """ Mapnik Singleton to allow for just a single import of mapnik,
    either old mapnik or mapnik2.
    
    For allowing nik2img full control over version based on command line arg.
    
    Also, to avoid pitfall of double import:

    >>> import mapnik
    >>> import mapnik2
    Traceback (most recent call last):
      File "<stdin>", line 1, in <module>
      File "/Library/Python/2.5/site-packages/mapnik2/__init__.py", line 54, in <module>
        from _mapnik2 import *
    RuntimeError: unidentifiable C++ exception
    
    Oh Singletons: http://lucumr.pocoo.org/2009/7/24/singletons-and-their-problems-in-python
    
    """
    
    class __impl:
        """Single Mapnik instance"""
        
        def get_version(self):
            return self.version 

    __instance = None
    __version = None

    def __init__(self,version=None):
        if Mapnik.__instance is None:
            if version in (None, 2):
                if version is None:
                    try:
                        import mapnik2
                        Mapnik.__instance = mapnik2
                    except:
                        import mapnik
                        Mapnik.__instance = mapnik
                else:
                    import mapnik2
                    Mapnik.__instance = mapnik2
            else:
                import mapnik
                Mapnik.__instance = mapnik
            
            
            # only major breakage in python bindings with mapnik2 (so far)
            if not hasattr(Mapnik.__instance,'Envelope'):
                Mapnik.__instance.Envelope = Mapnik.__instance.Box2d

            # bring 0.5.x series python bindings up to 0.6.x
            if not hasattr(Mapnik.__instance,'mapnik_version'):
                from mapnik_utils import compatibility 

            # extent mapnik objects 
            from mapnik_utils import metaclass_injectors

            Mapnik.__version = version

        # Store instance reference as the only member in the handle
        self.__dict__['_Mapnik__instance'] = Mapnik.__instance
        

    def __getattr__(self, attr):
        return getattr(self.__instance, attr)

    def __setattr__(self, attr, value):
        return setattr(self.__instance, attr, value)